﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;

namespace TestProject
{
    /// <summary>
    /// 주문 데이터 리스트 소스
    /// </summary>
    public class OrderDataListSource : IListSource
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 잠금자
        /// </summary>
        private static object _locker = new object();

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 주문 데이터 리스트
        /// </summary>
        private List<OrderData> orderDataList;

        /// <summary>
        /// 카운트
        /// </summary>
        private int count = 0;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 리스트 컬렉션 포함 여부 - ContainsListCollection

        /// <summary>
        /// 리스트 컬렉션 포함 여부
        /// </summary>
        public bool ContainsListCollection
        {
            get
            {
                return false;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - OrderDataListSource(count)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="count">카운트</param>
        public OrderDataListSource(int count)
        {
            this.count = count;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 리스트 구하기 - GetList()

        /// <summary>
        /// 리스트 구하기
        /// </summary>
        /// <returns>리스트</returns>
        public IList GetList()
        {
            if(this.orderDataList == null)
            {
                lock(_locker)
                {
                    if(this.orderDataList == null)
                    {
                        GenerateOrderData();
                    }
                }
            }

            return this.orderDataList;
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 주문 데이터 생성하기 - GenerateOrderData()

        /// <summary>
        /// 주문 데이터 생성하기
        /// </summary>
        private void GenerateOrderData()
        {
            this.orderDataList = new List<OrderData>(this.count);

            Random random = new Random();

            int customerCount = Data.CustomerList.Count;
            int productCount  = Data.ProductList.Count;

            for(int i = 0; i < this.count; i++)
            {
                OrderData orderData = new OrderData();

                orderData.OrderID      = i + 1;
                orderData.CustomerName = Data.CustomerList[random.Next(customerCount)];
                orderData.OrderDate    = DateTime.Today.Subtract(TimeSpan.FromDays(random.Next(180)));

                KeyValuePair<string, decimal> productKeyValuePair = Data.ProductList[random.Next(productCount)];

                orderData.ProductName = productKeyValuePair.Key;
                orderData.Price       = productKeyValuePair.Value;
                orderData.Quantity    = random.Next(200) + 1;

                this.orderDataList.Add(orderData);
            }
        }

        #endregion
    }
}