﻿using System.ComponentModel;

namespace TestProject
{
    /// <summary>
    /// 뷰 모델
    /// </summary>
    public class ViewModel : INotifyPropertyChanged
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Event
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 속성 변경시 - PropertyChanged

        /// <summary>
        /// 속성 변경시
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 주문 데이터 리스트 소스
        /// </summary>
        private OrderDataListSource orderDataListSource;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 주문 데이터 리스트 소스 - OrderDataListSource

        /// <summary>
        /// 주문 데이터 리스트 소스
        /// </summary>
        public OrderDataListSource OrderDataListSource
        {
            get
            {
                return this.orderDataListSource;
            }
            set
            {
                if(this.orderDataListSource == value)
                {
                    return;
                }

                this.orderDataListSource = value;

                OnPropertyChanged("OrderDataListSource");
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - ViewModel()

        /// <summary>
        /// 생성자
        /// </summary>
        public ViewModel()
        {
            OrderDataListSource = new OrderDataListSource(1000000);
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 속성 변경시 처리하기 - OnPropertyChanged(propertyName)

        /// <summary>
        /// 속성 변경시 처리하기
        /// </summary>
        /// <param name="propertyName">속성명</param>
        protected virtual void OnPropertyChanged(string propertyName)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }

        #endregion
    }
}